from abc import ABC, abstractmethod
from typing import Dict, List, Optional

from pythonit_toolkit.emails.templates import EmailTemplate


class EmailBackend(ABC):
    def __init__(self, environment: Optional[str] = None) -> None:
        self.environment = environment

    @abstractmethod
    def send_email(
        self,
        *,
        template: EmailTemplate,
        subject: str,
        from_: str,
        to: str,
        variables: Optional[Dict[str, str]] = None,
        reply_to: List[str] = None
    ):
        pass



from dataclasses import dataclass
import importlib

from .backends.base import EmailBackend

EMAIL_BACKEND_CACHE: dict[str, EmailBackend] = {}


def get_email_backend(backend_path: str, **options: dict[str, str]) -> EmailBackend:
    global EMAIL_BACKEND_CACHE

    instance = EMAIL_BACKEND_CACHE.get(backend_path, None)

    if not instance:
        module_path, class_name = backend_path.rsplit(".", 1)
        module = importlib.import_module(module_path)
        class_impl = getattr(module, class_name)
        instance = class_impl(**options)
        EMAIL_BACKEND_CACHE[backend_path] = instance

    return instance


@dataclass
class SafeString(str):
    original_str: str

    def __str__(self) -> str:
        return self.original_str


def mark_safe(string: str) -> SafeString:
    if string is None:
        raise ValueError("string cannot be None")

    return SafeString(string)



from datetime import datetime, timedelta, timezone
from typing import Any

import jwt


def decode_pastaporto(token: str, secret: str) -> dict[str, Any]:
    return jwt.decode(
        token,
        str(secret),
        issuer="users",
        algorithms=["HS256"],
        options={"require": ["exp", "iss", "iat"]},
    )


def decode_service_to_service_token(
    token: str, secret: str, *, issuer: str, audience: str
):
    return jwt.decode(
        token,
        secret,
        audience=audience,
        issuer=issuer,
        algorithms=["HS256"],
        options={"require": ["exp", "iss", "aud", "iat"]},
    )


def generate_service_to_service_token(secret: str, issuer: str, audience: str):
    if not secret:
        raise ValueError("Secret can not be empty")

    now = datetime.now(tz=timezone.utc)

    return jwt.encode(
        {
            "iss": issuer,
            "aud": audience,
            "iat": now,
            "exp": now + timedelta(seconds=60),
        },
        str(secret),
        algorithm="HS256",
    )



import sentry_sdk
from sentry_sdk.integrations.aws_lambda import AwsLambdaIntegration


def strip_sensitive_data(event, hint):
    event["request"]["headers"].pop("x-pastaporto", "")
    return event


def configure_sentry(*, dsn: str, env: str):
    sentry_sdk.init(
        dsn=dsn,
        integrations=[AwsLambdaIntegration()],
        traces_sample_rate=0.1,
        environment=env,
        before_send=strip_sensitive_data,
    )



from sentry_sdk import add_breadcrumb
from strawberry.extensions import Extension


class SentryExtension(Extension):
    def on_request_start(self):
        add_breadcrumb(
            category="graphql",
            message=f"""
{self.execution_context.operation_name or 'No operation name'}
{self.execution_context.query}
            """.strip(),
            level="info",
        )
